#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "globals.h"
#include "choices.h"

#include "Icons/menu.xpm"
#include "Icons/menu2.xpm"

/* This function taken from the pixmaps.c file in ROX-Filer 2.0.1
   by Thomas Leonard. */
GdkPixbuf *scale_pixbuf(GdkPixbuf *src, int max_w, int max_h)
{
	int	w, h;
	w = gdk_pixbuf_get_width(src);
	h = gdk_pixbuf_get_height(src);

	if (w <= max_w && h <= max_h)
	{
		gdk_pixbuf_ref(src);
		return src;
	}
	else
	{
		float scale_x = ((float) w) / max_w;
		float scale_y = ((float) h) / max_h;
		float scale = MAX(scale_x, scale_y);
		int dest_w = w / scale;
		int dest_h = h / scale;
		
		return gdk_pixbuf_scale_simple(src,
						MAX(dest_w, 1),
						MAX(dest_h, 1),
						GDK_INTERP_BILINEAR);
	}
}

GdkPixbuf *scale_pixbuf_unconditional(GdkPixbuf *src, int max_w, int max_h)
{
	int	w, h;

	w = gdk_pixbuf_get_width(src);
	h = gdk_pixbuf_get_height(src);

	float scale_x = ((float) w) / max_w;
	float scale_y = ((float) h) / max_h;
	float scale = MAX(scale_x, scale_y);
	int dest_w = w / scale;
	int dest_h = h / scale;
		
	return gdk_pixbuf_scale_simple(src,
						MAX(dest_w, 1),
						MAX(dest_h, 1),
						GDK_INTERP_BILINEAR);
}

void change_menu_pixmap(gint state)
{
	gchar *ico;
	char ** def;

	if (GPOINTER_TO_INT(state) == ENTER) {
		ico = menu_file_on;
		def = menu2_xpm;
	} else {
		ico = menu_file_off;
		def = menu_xpm;
	}
	
   if (GTK_IS_WIDGET(_pixmap) )
		gtk_container_remove(GTK_CONTAINER(button), _pixmap);
	/*gtk_widget_destroy(GTK_WIDGET(_pixmap));*/
   _pixmap = create_scaled_pixmap(ico, def);
   gtk_widget_show(_pixmap);
   gtk_container_add(GTK_CONTAINER(button), _pixmap);
}

GtkWidget *create_scaled_pixmap(char *path, char ** default_icon)
{
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;
    GdkPixbuf *pixbuf, *pixbuf_scaled;
    struct stat info;

    if (!stat(path, &info)) {
#ifdef GTK1
	pixbuf = gdk_pixbuf_new_from_file(path);
#else
	pixbuf = gdk_pixbuf_new_from_file(path, NULL);
#endif
    } else {
	pixbuf = gdk_pixbuf_new_from_xpm_data((const char **) default_icon);
    }
   /*pixbuf_scaled = gdk_pixbuf_scale_simple(pixbuf, icon_width, 
	                icon_height, GDK_INTERP_BILINEAR);*/
	 pixbuf_scaled = scale_pixbuf_unconditional(pixbuf, icon_width, icon_height);
    gdk_pixbuf_render_pixmap_and_mask(pixbuf_scaled, &gdkpixmap,
				      &mask, 1);
    pixmap = gtk_pixmap_new(gdkpixmap, mask);
    gdk_pixmap_unref(gdkpixmap);
	 gdk_pixbuf_unref(pixbuf_scaled);
	 gdk_pixbuf_unref(pixbuf);
	 gdk_bitmap_unref(mask);
    
	 return pixmap;
}

/* Write the config to a file. */
void write_config(void) {

	gchar *fname;

	/* Use the choices system to get the name to save to */
	fname=choices_find_path_save("options", PROJECT, TRUE);

	if(fname) {
		FILE *out;

		out=fopen(fname, "w");
		if(out) {

			/* Write config here... */
			fprintf(out, ("icon_width = %d\n"), icon_width);
			fprintf(out, ("icon_height = %d\n"), icon_height);
			fprintf(out, ("menu_file_on = %s\n"), menu_file_off);
			fprintf(out, ("menu_file_off = %s\n"), menu_file_off);
      
			fclose(out);
		}

		g_free(fname);
	}
}
/* Read in the config. */
void read_config(void) {

	guchar *fname;

	/* Use the choices system to locate the file to read */
	fname=choices_find_path_load("options", PROJECT);

	if(fname) {
		FILE *in;
		in=fopen(fname, "r");
		if(in) {
			char buf[1024], *line;
			char *end, *start;
			int val;

			do {
				line=fgets(buf, sizeof(buf), in);
				if(!line)
					break;
				end=strchr(line, '\n');
				if(end)
					*end=0;
				end=strchr(line, '#');  /* everything after # is a comment */
				if(end)
					*end=0;

				/* Process the line here... */
				if( strstr(line, "icon_width") ) {
					
					start = strstr(line, "=");
					start = (char*) start+1;
					val = atoi(start);
					if (val > 0) icon_width = val;

				} else if ( strstr(line, "icon_height") ) {

					start = strstr(line, "=");
					start = (char*) start+1;
					val = atoi(start);
					if (val > 0) icon_height = val;

				} else if ( strstr(line, "menu_file_on") ) {

					start = strstr(line, "=");
					start = (char*) start+1;
					while (start[0] != '/' && start[0] != '\0')
						start = (char*) start+1;
					
					/*g_free(cdrom_mount_path);*/
					menu_file_on = g_strdup(start);
					
				} else if ( strstr(line, "menu_file_off") ) {

					start = strstr(line, "=");
					start = (char*) start+1;
					while (start[0] != '/' && start[0] != '\0')
						start = (char*) start+1;
					
					/*g_free(cdrom_mount_path);*/
					menu_file_off = g_strdup(start);
				}

			} while(!feof(in));

			fclose(in);
		}

		g_free(fname);
	}
}

